;;; Display Layer -*- lexical-binding: t; -*-

(setq display-packages
      '(;; Owned packages
        all-the-icons
        all-the-icons-ivy
        all-the-icons-dired
        pretty-mode
        ;;solarized-theme
        (prettify-utils :location (recipe :fetcher github
                                          :repo "Ilazki/prettify-utils.el"))

        ;; Elsehwere-owned packages
        spaceline-all-the-icons
        which-key

        ;; Personal display-related packages
        (pretty-code     :location local)
        (pretty-eshell   :location local)
        (pretty-fonts    :location local)
        (pretty-magit    :location local)
        (pretty-outlines :location local)))

;;; Owned Packages
;;;; All-the-icons

(defun display/post-init-all-the-icons ()
  (use-package all-the-icons
    :config
    (let ((hy-icon '(all-the-icons-fileicon "hy" :face all-the-icons-orange))
          (dt-icon '(all-the-icons-fileicon "graphviz" :face all-the-icons-pink)))
      (add-to-list 'all-the-icons-icon-alist      `("\\.hy$"          ,@hy-icon))
      (add-to-list 'all-the-icons-icon-alist      `("\\.dot$"         ,@dt-icon))
      (add-to-list 'all-the-icons-mode-icon-alist `(hy-mode           ,@hy-icon))
      (add-to-list 'all-the-icons-mode-icon-alist `(graphviz-dot-mode ,@dt-icon)))))

;;;; All-the-icons-ivy

(defun display/init-all-the-icons-ivy ()
  (use-package all-the-icons-ivy
    :config
    (progn
      ;; Fix icon prompt alignment in ivy prompts
      (advice-add 'all-the-icons-ivy-file-transformer :override
                  'all-the-icons-ivy-file-transformer-stdized)

      ;; Add behavior to counsel projectile funcs too
      (advice-add 'counsel-projectile-find-file-transformer :filter-return
                  'all-the-icons-ivy-file-transformer-stdized)
      (advice-add 'counsel-projectile-transformer :filter-return
                  'all-the-icons-ivy-file-transformer-stdized)

      (all-the-icons-ivy-setup))))

;;;; All-the-icons-dired

(defun display/init-all-the-icons-dired ()
  (use-package all-the-icons-dired
    :hook (dired-mode . all-the-icons-dired-mode)))

;;;; Pretty-mode

(defun display/init-pretty-mode ()
  ;; I *only* use greek letter replacements at the moment.
  ;; However, I go back and forht on whether to use nil-like <-> emptyset.
  ;; I currently have it *enabled*. Uncomment the deactivation to remove it.

  (use-package pretty-mode
    :config
    (progn
      (global-pretty-mode t)

      (pretty-deactivate-groups
       '(;;:equality :ordering :ordering-double :ordering-triple
                   ;;:arrows :arrows-twoheaded :punctuation
                   ;;:logic :sets
                   ;;;; :nil
                   ))
      (pretty-activate-groups
       '(:greek :equality :ordering :ordering-double :ordering-triple :arrows :arrows-twoheaded :punctuation :logic :sets :nil)))))

;;;; Prettify-utils

(defun display/init-prettify-utils ()
  (use-package prettify-utils))

;;;; Solarized-theme

;(defun display/init-solarized-theme ()
;  (use-package solarized-theme))

;;; Unowned Packages
;;;; Which-key

(defun display/post-init-which-key ()
  (when (configuration-layer/package-used-p 'pretty-fonts)
    (setq which-key-separator " ")
    (setq which-key-prefix-prefix " ")))

;;;; Spaceline-all-the-icons

(defun display/post-init-spaceline-all-the-icons ()
  (spaceline-all-the-icons-theme)

  ;;(setq spaceline-highlight-face-func 'spaceline-highlight-face-default)

  (setq spaceline-all-the-icons-icon-set-modified         'circle)
  (setq spaceline-all-the-icons-icon-set-window-numbering 'square)
  (setq spaceline-all-the-icons-separator-type            'none )
  ;;(setq spaceline-all-the-icons-primary-separator         "")


  (spaceline-toggle-all-the-icons-bookmark-on)
  (spaceline-toggle-all-the-icons-dedicated-on)
  (spaceline-toggle-all-the-icons-fullscreen-on)
  (spaceline-toggle-all-the-icons-buffer-position-on)
  (spaceline-toggle-all-the-icons-narrowed-on)
  (spaceline-toggle-all-the-icons-eyebrowse-workspace-on)
  (spaceline-toggle-all-the-icons-multiple-cursors-on)
  ;; !!!!!!!!!!!!!!!!
  ;; !! https://github.com/domtronn/spaceline-all-the-icons.el/issues/55
  ;; !! If you remove this - expect EXTREMELY degraded performance
  ;; !! on files of more-or-less any size and of any type
  ;; !!!!!!!!!!!!!!!!
  ;;(spaceline-toggle-projectile-root-off)
  ;;(spaceline-toggle-all-the-icons-projectile-off)
  ;;(spaceline-toggle-all-the-icons-buffer-id-off)


  ;; Mode Segments
  ;;(spaceline-toggle-all-the-icons-minor-modes-off)

  ;; Buffer Segments
  ;;(spaceline-toggle-all-the-icons-buffer-size-off)
  ;;(spaceline-toggle-all-the-icons-buffer-position-off)

  ;; Git Segments
  ;;(spaceline-toggle-all-the-icons-git-status-off)
  ;;(spaceline-toggle-all-the-icons-vc-icon-off)
  ;;(spaceline-toggle-all-the-icons-vc-status-off)

  ;; Misc Segments
  ;;(spaceline-toggle-all-the-icons-eyebrowse-workspace-off)
  ;;(spaceline-toggle-all-the-icons-flycheck-status-off)
  ;;(spaceline-toggle-all-the-icons-time-off)
  )

;;; Pretty Packages
;;;; Pretty-code

(defun display/init-pretty-code ()
  (use-package pretty-code
    :config
    (progn
      (pretty-code-add-hook 'emacs-lisp-mode-hook '((:def "defun")))
      (pretty-code-add-hook 'hy-mode-hook         '((:def "defn")
                                                    (:lambda "fn")))
      (pretty-code-add-hook 'python-mode-hook     '((:def "def")
                                                    (:lambda "lambda")
                                                    (:null "None")
                                                    (:true "True")
                                                    (:false "False")
                                                    (:int "int")
                                                    (:float "float")
                                                    (:str "str")
                                                    (:bool "bool")
                                                    (:not "not")
                                                    (:for "for")
                                                    (:in "in")
                                                    (:not-in "not in")
                                                    (:return "return")
                                                    (:yield "yield")
                                                    (:and "and")
                                                    (:or "or")
                                                    (:tuple "Tuple")
                                                    (:pipe "tz-pipe")))
        (setq prettify-symbols-unprettify-at-point 'right-edge)
      )))

;;;; Pretty-eshell

(defun display/init-pretty-eshell ()
  (use-package pretty-eshell
    :init
    (progn
      ;; Change default banner message
      (setq eshell-banner-message (s-concat (s-repeat 20 "---") "\n\n"))

      ;; More prompt styling
      (setq pretty-eshell-header "\n┌─")
      (setq pretty-eshell-prompt-string "└─> "))

    :config
    (progn
      ;; Directory
      (pretty-eshell-section
       esh-dir
       "\xf07c"  ; 
       (abbreviate-file-name (eshell/pwd))
       '(:foreground "#268bd2" :bold bold :underline t))

      ;; Git Branch
      (pretty-eshell-section
       esh-git
       "\xe907"  ; 
       (magit-get-current-branch)
       '(:foreground "#8D6B94"))

      ;; Python Virtual Environment
      (pretty-eshell-section
       esh-python
       "\xe928"  ; 
       pyvenv-virtual-env-name)

      ;; Time
      (pretty-eshell-section
       esh-clock
       "\xf017"  ; 
       (format-time-string "%H:%M" (current-time))
       '(:foreground "forest green"))

      ;; Prompt Number
      (pretty-eshell-section
       esh-num
       "\xf0c9"  ; 
       (number-to-string pretty-eshell-prompt-num)
       '(:foreground "brown"))

      (setq pretty-eshell-funcs
            (list esh-dir esh-git esh-python esh-clock esh-num)))))

;;;; Pretty-fonts

(defun display/init-pretty-fonts ()
  (use-package pretty-fonts
    :config
    ;; !! This is required to avoid segfault when using emacs as daemon !!
    (spacemacs|do-after-display-system-init
     ;; (pretty-fonts-add-hook 'prog-mode-hook pretty-fonts-fira-code-alist)
     ;; (pretty-fonts-add-hook 'org-mode-hook  pretty-fonts-fira-code-alist)

     ;; (pretty-fonts-set-fontsets-for-fira-code)
     (pretty-fonts-set-fontsets
      '(;; All-the-icons fontsets
        ("fontawesome"
         ;;                         
         #xf07c #xf0c9 #xf0c4 #xf0cb #xf017 #xf101)

        ("all-the-icons"
         ;;    
         #xe907 #xe928)

        ("github-octicons"
         ;;                               
         #xf091 #xf059 #xf076 #xf075 #xe192  #xf016 #xf071)

        ("material icons"
         ;;              
         #xe871 #xe918 #xe3e7  #xe5da
         ;;              
         #xe3d0 #xe3d1 #xe3d2 #xe3d4))))))

;;;; Pretty-magit

;;(defun display/init-pretty-magit ()
;;  (use-package pretty-magit
;;    :config
;;    (progn
;;      (pretty-magit-add-leaders
;;       '(("Feature" ? (:foreground "slate gray" :height 1.2))
;;         ("Add"     ? (:foreground "#375E97" :height 1.2))
;;         ("Fix"     ? (:foreground "#FB6542" :height 1.2))
;;         ("Clean"   ? (:foreground "#FFBB00" :height 1.2))
;;         ("Docs"    ? (:foreground "#3F681C" :height 1.2))
;;         ("master"  ? (:box nil :height 1.0 :family "github-octicons"))
;;         ("origin"  ? (:box nil :height 1.0 :family "github-octicons"))))
;;
;;      (pretty-magit-setup))))

(defun display/init-pretty-magit ()
  (use-package pretty-magit
    :after magit
    :config
    (progn
      (pretty-magit "Release"  ? (:foreground "gold" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Git"      ? (:foreground "blue" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Chore"    ? (:foreground "golden rod" :height 1.0 :family "FontAwesome"))
      (pretty-magit "CI"       ? (:foreground "green" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Test"     ? (:foreground "magenta" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Refactor" ? (:foreground "olive" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Style"    ? (:foreground "purple" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Build"    ? (:foreground "silver" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Perf"     ? (:foreground "red" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Feat"     ? (:foreground "#375E97" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Fix"      ? (:foreground "#FB6542" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Revert"   ? (:foreground "#FFBB00" :height 1.0 :family "FontAwesome"))
      (pretty-magit "Docs"     ? (:foreground "#3F681C" :height 1.0 :family "FontAwesome"))
      (pretty-magit "master"   ? (:box nil :height 1.0 :family "github-octicons") t)
      (pretty-magit "origin"   ? (:box nil :height 1.0 :family "github-octicons") t)
      )))

;;;; Pretty-outlines

(defun display/init-pretty-outlines ()
  (use-package pretty-outlines
    :hook ((outline-mode       . pretty-outlines-set-display-table)
           (outline-minor-mode . pretty-outlines-set-display-table)
           (emacs-lisp-mode . pretty-outlines-add-bullets)
           (hy-mode         . pretty-outlines-add-bullets)
           (python-mode     . pretty-outlines-add-bullets))))
